package cn.com.ty.lift.business.common;

import cn.com.ty.lift.business.annualinspection.service.AnnualInspectionService;
import cn.com.ty.lift.business.emergency.service.EmergencyRepairService;
import cn.com.ty.lift.business.framework.conf.SystemConfiguration;
import cn.com.ty.lift.business.framework.util.MessageUtils;
import cn.com.ty.lift.business.library.dao.entity.LiftBrand;
import cn.com.ty.lift.common.constants.RedisConstants;
import cn.com.ty.lift.common.model.AreaCode;
import cn.com.ty.lift.common.utils.ValuePool;
import cn.com.ty.lift.common.verify.Ver;
import cn.com.ty.lift.common.verify.Verifier;
import cn.com.ty.lift.common.verify.Verify;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.*;

/**
 * @author bieao
 * @date 2019/12/19
 * @description 公共初始化数据
 */
@Slf4j
@AllArgsConstructor
@RestController
@RequestMapping("common")
public class CommonController {

    private static final String DateInPath = "yyyy/MM/dd";

    private RedisTemplate<String, Object> redisTemplate;
    private SystemConfiguration           systemConfiguration;
    private AnnualInspectionService       annualInspectionService;
    private EmergencyRepairService        emergencyRepairService;

    /**
     * @return RestResponse 省、市、区树形结构列表
     * @description 查询省、市、区树形结构列表
     * @date 2019/12/4 10:23 AM
     */
    @PostMapping("area/list")
    @SuppressWarnings("unchecked")
    public RestResponse areaList() {
        List<AreaCode> areaCodeList = (List<AreaCode>)redisTemplate.opsForValue().get(RedisConstants.RK_AREA_LIST);
        if (ObjectUtil.isEmpty(areaCodeList)) {
            return RestResponse.success();
        }
        return RestResponse.success(areaCodeList, MessageUtils.get("msg.query.success"));
    }

    /**
     * @return RestResponse 电梯品牌列表
     * @description 查询电梯品牌列表
     * @date 2019/12/10 10:23 AM
     */
    @PostMapping("brand/list")
    @SuppressWarnings(value = "unchecked")
    public RestResponse list(){
        List<LiftBrand> liftBrandList = (List<LiftBrand>) redisTemplate.opsForValue().get(RedisConstants.RK_LIFT_BRAND_LIST);
        if (ObjectUtil.isEmpty(liftBrandList)) {
            return RestResponse.success();
        }
        return RestResponse.success(liftBrandList, MessageUtils.get("msg.query.success"));
    }

    /**
     * 批量上传多个文件
     *
     * @param files 文件列表
     * @return RestResponse
     */
    @PostMapping("uploads")
    public RestResponse uploads(@RequestParam("files") MultipartFile[] files){
        Verify.notTrue(Objects.isNull(files) || files.length == 0,Verify.Upload.fileDataMissing);
        try {
            Map<String, MultipartFile> fileMap = new HashMap<>();
            //1 先解析文件格式
            for (MultipartFile file : files) {
                String fileName = handleFile(file);
                if(StrUtil.isEmpty(fileName)){
                    return RestResponse.fail(Verify.Upload.fileFormatNotSupport);
                }
                fileMap.put(fileName, file);
            }
            //2 批量上传
            List<String> urls = new ArrayList<>();
            for (Map.Entry<String, MultipartFile> entry : fileMap.entrySet()){
                String url = systemConfiguration.build().putObject(entry.getKey(), entry.getValue().getBytes());
                log.info("上传文件，文件URL: {}",url);
                Verify.notNull(url, Verify.Upload.fileUploadFail);
                urls.add(url);
            }
            return RestResponse.success(urls);
        }catch (Exception e){
            log.error("上传文件异常",e);
            return RestResponse.fail(Verify.Upload.fileUploadFail);
        }
    }

    /**
     * 上传单个文件
     *
     * @param file 上传的文件
     * @return RestResponse
     */
    @PostMapping("upload")
    public RestResponse upload(@RequestParam("file") MultipartFile file){
        Verify.notTrue(Objects.isNull(file) || file.isEmpty(),Verify.Upload.fileDataMissing);
        try {
            String fileName = handleFile(file);
            if(StrUtil.isEmpty(fileName)){
                return RestResponse.fail(Verify.Upload.fileFormatNotSupport);
            }
            String url = systemConfiguration.build().putObject(fileName, file.getBytes());
            log.info("上传文件，文件URL: {}",url);
            return RestResponse.success(url);
        }catch (Exception e){
            log.error("上传文件异常",e);
            return RestResponse.fail(Verify.Upload.fileUploadFail);
        }
    }

    /**
     * 校验文件大小，文件格式，生成新文件名
     *
     * @param file the {@link MultipartFile} to upload
     * @return the name string of the file.
     */
    private String handleFile(MultipartFile file){
        Verify.notTrue(Objects.isNull(file) || file.isEmpty(),Verify.Upload.fileDataMissing);
        // 获取文件名，带后缀
        String originalFilename = file.getOriginalFilename();
        log.info("上传文件，原文件名：{}", originalFilename);
        // 获取文件的后缀格式
        int lastDotIndex = originalFilename.lastIndexOf(ValuePool.DOT);
        Verify.notTrue(-1 == lastDotIndex, "文件名解析不到文件格式");
        String fileSuffix = originalFilename.substring(lastDotIndex).toLowerCase();
        Verify.notNull(fileSuffix,Verify.Upload.fileFormatIllegal);
        long fileSize = file.getSize();
        if(StrUtil.equalsAny(fileSuffix,Verify.Upload.pics)){
            Verify.notTrue(fileSize > Verify.Upload.maxSizePic,"文件大小不超过" + Verify.Upload.maxSizePicDesc);
        }else if(StrUtil.equalsAny(fileSuffix,Verify.Upload.files)){
            Verify.notTrue(fileSize > Verify.Upload.maxSizeFile,"文件大小不超过" + Verify.Upload.maxSizeFileDesc);
        }else if(StrUtil.equalsAny(fileSuffix,Verify.Upload.videos)){
            Verify.notTrue(fileSize > Verify.Upload.maxSizeVideo,"文件大小不超过" + Verify.Upload.maxSizeVideoDesc);
        }else{
            return null;
        }
        String fileName = StrUtil.format("{}/{}{}",DateUtil.format(DateUtil.date(),DateInPath),IdWorker.getIdStr(),fileSuffix);
        log.info("上传文件，新文件名：{}", fileName);
        return fileName;
    }

    @PostMapping("countDoing")
    @Verifier(fields = {"mtCompanyId", "userId"})
    public RestResponse countDoing(@Ver @RequestBody CommonRequest request){
        long inspection = annualInspectionService.countDoingByUser(request);
        long repair = emergencyRepairService.countDoingByUser(request);
        CommonResponse common = new CommonResponse();
        common.setInspection(inspection);
        common.setRepair(repair);
        return RestResponse.success(common);
    }
}
